const Models = require('./../models/common');
const router = require('express').Router();
const helper = require('./../Helpers/handler');
const e = require('express');
const config = require('./../../config/database');
const jwt = require('jsonwebtoken');
const passport = require('passport')

router.post('/:term', (req, res) => {
    const label = req.params.term;
    const Table = getModel(label)[0];
    const limit = req.body.limit ? req.body.limit : 20;
    const page = req.body.page ? req.body.page : 1;
    const skip = (page * limit) - limit;
    const sort = req.body.sort ? req.body.sort : "_id";
    const sortQuery = {};
    sortQuery[sort] = req.body.reverse ? req.body.reverse : -1;
    if (Table) {
        Table.TableModel.countDocuments((err, docsLength) => {
            if (err) {
                helper.errHandler(res, err);
            } else {
                Table.TableModel.find((err, docs) => {
                    if (err) {
                        helper.errHandler(res, err);
                    } else {
                        res.json({
                            success: true,
                            msg: 'Data Fetched',
                            data: {
                                docs,
                                docsLength
                            }
                        });
                    }
                }).sort(sortQuery).skip(skip).limit(limit);
            }
        });
    } else {
        helper.errHandler(res, { message: 'Table Not Found' });
    }
});

router.post('/:term/query', (req, res) => {
    const label = req.params.term;
    const query = req.body.query;
    const Table = getModel(label)[0];
    const limit = req.body.limit ? req.body.limit : 20;
    const page = req.body.page ? req.body.page : 1;
    const skip = (page * limit) - limit;
    const sort = req.body.sort ? req.body.sort : "_id";
    const sortQuery = {};
    sortQuery[sort] = req.body.reverse ? req.body.reverse : -1;
    if (Table) {
        Table.TableModel.countDocuments(query, (err, docsLength) => {
            if (err) {
                helper.errHandler(res, err);
            } else {
                Table.TableModel.find(query, (err, docs) => {
                    if (err) {
                        helper.errHandler(res, err);
                    } else {
                        res.json({
                            success: true,
                            msg: 'Data Fetched',
                            data: {
                                docs,
                                docsLength
                            }
                        });
                    }
                }).sort(sortQuery).skip(skip).limit(limit);
            }
        });
    } else {
        helper.errHandler(res, { message: 'Table Not Found' });
    }
});

router.post('/:term/search', (req, res) => {
    const label = req.params.term;
    const query = req.body.query;
    const Table = getModel(label)[0];
    const limit = req.body.limit ? req.body.limit : 20;
    const page = req.body.page ? req.body.page : 1;
    const skip = (page * limit) - limit;
    for (const key of Object.keys(query)) {
        if (
            key != "_id" &&
            key != "markDateFrom" &&
            key != "markDateTo"
        ) {
            query[key] = new RegExp(query[key], 'gi');
        } else {
            if (key == "markDateFrom") {
                query['$and'] = [{ "created_at": query["markDateFrom"] }, { "created_at": query["markDateTo"] }];
                delete query["markDateFrom"];
                delete query["markDateTo"];
            } else if (key == "markDateTo") {
                continue;
            } else {
                query[key] = query[key];
            }
        }
    }
    if (Table) {
        Table.TableModel.countDocuments(query, (err, docsLength) => {
            if (err) {
                helper.errHandler(res, err);
            } else {
                Table.TableModel.find(query, (err, docs) => {
                    if (err) {
                        helper.errHandler(res, err);
                    } else {
                        res.json({
                            success: true,
                            msg: 'Data Fetched',
                            data: {
                                docs,
                                docsLength
                            }
                        });
                    }
                }).sort({ _id: -1 }).skip(skip).limit(limit);
            }
        });
    } else {
        helper.errHandler(res, { message: 'Table Not Found' });
    }
});

router.post('/:term/create', async (req, res) => {
    const label = req.params.term;
    const doc = req.body.doc;
    doc.created_at = Date.now();
    const Table = getModel(label)[0];
    if (label == 'amsattendance') {
        const fs = require('fs');
        const folder_name = req.body.doc.username;
        const isExist = fs.existsSync('./attendanceLogs/' + folder_name);
        if (!isExist) {
            fs.mkdirSync('./attendanceLogs/' + folder_name);
        }
        fs.copyFileSync('./check/' + folder_name + '.jpg', './attendanceLogs/' + folder_name + '/' + doc.created_at + '.jpg');

        doc.imgSrc = folder_name + '/' + doc.created_at + '.jpg';
    }

    if (!doc) {
        return helper.errHandler(res, { message: "Can't insert blank entry" });
    }

    if (Table) {
        Table.TableModel.create(doc, (err, doc) => {
            if (err) {
                helper.errHandler(res, err);
            } else {
                res.json({
                    success: true,
                    msg: 'Data Created',
                    data: {
                        doc
                    }
                });
            }
        })
    } else {
        helper.errHandler(res, { message: 'Table Not Found' });
    }
});

router.post('/:term/bulkcreate', (req, res) => {
    const label = req.params.term;
    const docs = req.body.docs;
    const Table = getModel(label)[0];

    if (!docs) {
        return helper.errHandler(res, { message: "Can't insert blank entry" });
    }
    if (!docs.length) {
        return helper.errHandler(res, { message: "Can't insert blank entry" });
    }

    if (Table) {
        Table.TableModel.insertMany(docs, (err, docs) => {
            if (err) {
                helper.errHandler(res, err);
            } else {
                res.json({
                    success: true,
                    msg: 'Data Created',
                    data: {
                        docsCount: docs.length
                    }
                });
            }
        })
    } else {
        helper.errHandler(res, { message: 'Table Not Found' });
    }
});

router.post('/:term/update', (req, res) => {
    const label = req.params.term;
    const query = req.body.query;
    const doc = req.body.doc;
    const Table = getModel(label)[0];
    if (Table) {
        Table.TableModel.updateOne(query, { $set: doc }, (err, doc) => {
            if (err) {
                helper.errHandler(res, err);
            } else {
                res.json({
                    success: true,
                    msg: 'Data Updated',
                    data: {
                        doc
                    }
                });
            }
        })
    } else {
        helper.errHandler(res, { message: 'Table Not Found' });
    }
});

router.post('/:term/updatebyid', (req, res) => {
    const label = req.params.term;
    const id = req.body.id;
    const doc = req.body.doc;

    const Table = getModel(label)[0];
    if (Table) {
        Table.TableModel.findByIdAndUpdate(id, { $set: doc }, (err, doc) => {
            if (err) {
                helper.errHandler(res, err);
            } else {
                res.json({
                    success: true,
                    msg: 'Data Updated',
                    data: {
                        doc
                    }
                });
            }
        })
    } else {
        helper.errHandler(res, { message: 'Table Not Found' });
    }
});

router.post('/:term/delete', (req, res) => {
    const label = req.params.term;
    const query = req.body.query;
    const Table = getModel(label)[0];
    if (Table) {
        Table.TableModel.find(query, (err, docs) => {
            if (err) { helper.errHandler(res, err) }
            else {
                Table.OldTableModel.insertMany(docs, (err, docs) => {
                    if (err) { helper.errHandler(res, err) }
                    else {
                        Table.TableModel.deleteMany(query, (err, doc) => {
                            if (err) {
                                helper.errHandler(res, err);
                            } else {
                                res.json({
                                    success: true,
                                    msg: 'Data Deleted',
                                    data: {
                                        doc
                                    }
                                });
                            }
                        });
                    }
                });
            }
        })
    } else {
        helper.errHandler(res, { message: 'Table Not Found' });
    }
});

router.post('/:term/deletebyid', (req, res) => {
    const label = req.params.term;
    const id = req.body.id;
    const Table = getModel(label)[0];
    if (Table) {
        Table.TableModel.findById(id, (err, doc) => {
            if (err) { helper.errHandler(res, err) }
            else {
                Table.OldTableModel.insertMany(doc, (err, doc) => {
                    if (err) { helper.errHandler(res, err) }
                    else {
                        Table.TableModel.findByIdAndDelete(id, (err, doc) => {
                            if (err) {
                                helper.errHandler(res, err);
                            } else {
                                res.json({
                                    success: true,
                                    msg: 'Data Deleted',
                                    data: {
                                        doc
                                    }
                                });
                            }
                        });
                    }
                });
            }
        });
    } else {
        helper.errHandler(res, { message: 'Table Not Found' });
    }
});

router.get('/:term/updateBranchAndBatch', (req, res) => {
    const label = req.params.term;
    const Table = getModel(label)[0];
    if (Table && Table.TableName == 'Student') {
        Table.TableModel.find((err, docs) => {
            if (err) { helper.errHandler(res, err) }
            else {
                res.json({ success: true, msg: "Updating....." });
                for (const doc of docs) {
                    doc.branch_name = doc.enrollment_no.substr(4, 2);
                    doc.batch_code = doc.course + '-' + doc.enrollment_no.substr(4, 2);
                    Table.TableModel.findOneAndUpdate({ _id: doc._id }, { $set: doc }, (err, doc) => {
                        if (err) {
                            console.log(err);
                        } else {
                            console.log("updated");
                        }
                    });
                }
            }
        });
    } else {
        helper.errHandler(res, { message: 'Table Not Found' });
    }
});

router.post('/:term/auth',
    (req, res) => {

        const query = req.body.query;

        if ((typeof (query)) == "object") {
            if (!query.username || !query.password || (typeof (query.username)) != 'string' || (typeof (query.password)) != 'string') {
                return res.json({
                    success: false,
                    msg: "Username OR Password is missing",
                    data: {
                        status: false,
                        docsLength: 0
                    }
                });
            }
        } else {
            return res.json({
                success: false,
                msg: "Username OR Password with query is missing",
                data: {
                    status: false,
                    docsLength: 0
                }
            });
        }

        const Table = getModel('amsuser')[0];

        if (Table) {
            Table.TableModel.findOne({
                username: query.username,
                password: query.password,
            }, (err, doc) => {
                if (err) {
                    helper.errHandler(res, err);
                } else {
                    if (!doc) {
                        return res.json({
                            success: false,
                            msg: 'Authenticaton Failed!',
                            data: {
                                status: false
                            }
                        });
                    } else {

                        var t_user = {
                            id: doc._id,
                            username: doc.username,
                            name: doc.name
                        };
                        const token = jwt.sign(t_user, config.secret, {
                            expiresIn: 10800
                        });

                        return res.json({
                            success: true,
                            msg: 'Data Fetched',
                            data: {
                                status: true,
                                token
                            }
                        });
                    }
                }
            }).maxTimeMS(20000);
        } else {
            helper.errHandler(res, { message: 'Table Not Found' });
        }
    });

module.exports = router;

// Handlers
function getModel(term) {
    const Model = Models.filter(o => o.TableName.toLowerCase() == term);
    return Model;
}
