const mongoose = require('mongoose');
const config = require('../config/database');

const ActivitySchema = mongoose.Schema({
    activityType: {
        type: String
    },
    activityData: {
        type: JSON
    },
    User: {
        type: JSON
    },
    UserId: {
        type: String
    },
    LeadId: {
        type: String
    },
    msg: {
        type: String
    },
    createdAt: {
        type: Date
    }
});

const Activity = (module.exports = mongoose.model('activity', ActivitySchema));

module.exports.addActivity = (newActivity, callback) => {
    newActivity.createdAt = Date.now();
    newActivity.save(callback);
};

module.exports.getActivityById = (id, usernames, callback) => {
    Activity.findOne({ $and: [{ _id: id }, { "User.username": { $in: usernames } }] }, callback);
};

module.exports.getAllActivities = (skip, usernames, callback) => {
    // console.log(usernames);
    Activity.find({ "User.username": { $in: usernames } }, callback).sort("-createdAt").skip(skip).limit(2000);
};

module.exports.searchInActivities = (keyword, skip, usernames, callback) => {
    Activity.find({
        $and: [{
            $or: [
                { activityType: keyword },
                { msg: keyword },
                { "activityData.lead.name": keyword },
                { "activityData.lead.mobile": keyword },
                { "activityData.newLead.name": keyword },
                { "activityData.newLead.mobile": keyword },
                { "User.username": keyword },
            ]
        }, { "User.username": { $in: usernames } }]
    }, callback).sort("-createdAt").skip(skip).limit(20);
};
