const config = require('../config/database');
const mongoose = require('mongoose');

const EventSchema = mongoose.Schema({
    Title: {
        type: String
    },
    hindiTitle: {
        type: String
    },
    shortTitle: {
        type: String
    },
    hindiShortTitle: {
        type: String
    },
    description: {
        type: String
    },
    hindiDescription: {
        type: String
    },
    live: {
        type: Boolean
    },
    running: {
        type: Boolean
    },
    religion: {
        type: JSON
    },
    caste: {
        type: JSON
    },
    incomeRange: {
        type: JSON
    },
    area: {
        type: JSON
    },
    limit: {
        type: String
    },
    softBookMrp: {
        type: String
    },
    softBookPrice: {
        type: String
    },
    hardBookMrp: {
        type: String
    },
    hardBookPrice: {
        type: String
    },
    pages: {
        type: Array,
        dafault: []
    },
    sortedMembersForBook: {
        type: Array,
        default: []
    },
    bookData: {
        type: JSON
    },
    firstCategory: {
        type: JSON,
        default: {
            title: '',
            values: []
        }
    },
    secondCategory: {
        type: JSON,
        default: {
            title: '',
            values: []
        }
    },
    thirdCategory: {
        type: JSON,
        default: {
            title: '',
            values: []
        }
    },
    startedAt: {
        type: Date
    },
    endedAt: {
        type: Date
    },
    createdAt: {
        type: Date,
        default: Date.now()
    },
    members: {
        type: Array,
        default: []
    },
    bookGenerated: {
        type: Boolean,
        default: false
    },
    updatedAt: {
        type: Date
    },
}, { collection: 'ss_events' });

const Event = (module.exports = mongoose.model('ss_events', EventSchema));

// Create and update
module.exports.addEvent = (newEvent, callback) => {
    newEvent.save(callback);

};

// get  
module.exports.getEvent = (id, callback) => {
    Event.findById(id, callback);
};

// get  
module.exports.getEventsByIds = (ids, callback) => {
    Event.find(
        {
            _id: {
                $in: ids
            }
        }, callback);
};

// get all 
module.exports.getAllEvents = (callback) => {
    Event.find(callback);
};

// get all upcoming events
module.exports.getAllUpcomingEvents = (callback) => {
    // let todayDate = Date.now();
    // todayDate.setDate(todayDate.getDate() + 1)
    var days = 1;
    var todayDate = Date.now() - days * 24 * 60 * 60 * 1000;
    Event.find(callback);
    // Event.find(
    //     {
    //         $or: [
    //             {
    //                 live: true
    //             },
    //             {
    //                 endedAt: {
    //                     $gte: todayDate
    //                 }
    //             }
    //         ]
    //     }, callback);
};

// get all ended events
module.exports.getAllEndedEvents = (callback) => {
    const todayDate = Date.now();
    Event.find(
        {
            $and: [
                {
                    live: false
                },
                {
                    endedAt: {
                        $lt: todayDate
                    }
                }
            ]
        }, callback);
};

// Update Book 
module.exports.updateBook = (id, pagesData, callback) => {
    Event.findByIdAndUpdate(id, { $set: { pages: pagesData, bookGenerated: true } }, callback);
};

// Update Book Sorted Members 
module.exports.updateBookSortedMembers = (id, sortedMembers, callback) => {
    Event.findByIdAndUpdate(id, { $set: { sortedMembersForBook: sortedMembers, bookGenerated: false } }, callback);
};

//delete 
module.exports.deleteEvent = (id, callback) => {
    Event.findByIdAndDelete(id, callback);
};
