const mongoose = require("mongoose");

const TableName = "ServiceRemainder";

const TableSchema = mongoose.Schema({
    alarmDate: {
        type: Date,
        index: true
    },
    alarmType: {
        type: String,
        trim: true
    },
    alarmForLead: {
        type: JSON
    },
    alarmSetBy: {
        type: String,
        trim: true
    },
    read: {
        type: String,
        default: 'N'
    },
    remark: {
        type: String,
        trim: true
    },
    status: {
        type: String,
        default: "Y"
    },
    created_at: {
        type: Date,
        index: true
    },
    created_by: String,
    updated_at: {
        type: Date,
        index: true
    },
    updated_by: String
});

const Table = (module.exports = mongoose.model(TableName, TableSchema));

const OldTable = mongoose.model("old" + TableName, TableSchema);

module.exports.addRow = (newRow, callback) => {
    newRow.created_at = Date.now();
    newRow.save(callback);
};

module.exports.updateRow = (id, newData, callback) => {
    newData.updated_at = Date.now();
    Table.findByIdAndUpdate(id, { $set: newData }, callback);
};

module.exports.getData = callback => {
    Table.find(callback);
};

module.exports.getDataById = (id, callback) => {
    Table.findById(id, callback);
};

module.exports.getDataByFieldName = (fieldName, fieldValue, callback) => {
    let query = {};
    query[fieldName] = fieldValue;
    Table.find(query, callback);
};

module.exports.getDataByFieldNames = (fieldNames, fieldValues, callback) => {
    let query = {};
    for (let i = 0; i < fieldNames.length; i++) {
        const fieldName = fieldNames[i];
        const fieldValue = fieldValues[i];
        query[fieldName] = fieldValue;
    }
    Table.find(query, callback);
};

module.exports.deleteTableById = (id, callback) => {
    Table.findById(id, (err, doc) => {
        if (err) {
            callback(err, null);
        } else {
            if (!doc) {
                callback(err, doc);
            } else {
                const dataToDel = new OldTable(doc);
                OldTable.insertMany(doc)
                    .then(val => {
                        Table.findByIdAndDelete(id, callback);
                    })
                    .catch(reason => {
                        callback(reason, null);
                    });
            }
        }
    });
};

