const mongoose = require('mongoose');
const config = require('../config/database');

const PackageSchema = mongoose.Schema({
  packageName: { type: String },
  shortName: { type: String },
  packageAmount: { type: String },
  purpose: { type: JSON },
  profileLimit: { type: String },
  balance: { type: String, default: 0 },
  source: { type: Array, default: [] },
  serviceType: { type: String },
  cities: { type: Array, default: [] },
  createdBy: { type: String },
  incomeRange: { type: JSON },
  unit: String,
  rate: Number,
  discount: Number,
  quantity: Number,
  additionalCharges: Number,
  packageType: String,
  renewDate: Date,
  serviceCount: String,
  serviceGap: String,
  billingCount: String,
  billingGap: String,
  createdAt: {
    type: Date,
    default: Date.now()
  },
  updatedAt: { type: Date }
});

const Package = (module.exports = mongoose.model('leadpackage', PackageSchema));

module.exports.addPackage = (newPackage, callback) => {
  newPackage.save(callback);
};

module.exports.getPackageByName = (packageName, callback) => {
  Package.findOne({ packageName: packageName }, callback);
};

module.exports.getPackageById = (id, callback) => {
  Package.findOne({ _id: id }, callback);
};

module.exports.getPackagesByFields = (source, purpose, cities, callback) => {
  Package.find({ $and: [{ source: { $in: source } }, { purpose: purpose }, { $or: [{ "cities.city": { $in: cities } }, { "cities.state": { $in: cities } }, { "cities.district": { $in: cities } }, { "cities.country": { $in: cities } },] }] }, { cities: 0 }, callback);
};

module.exports.deletePackage = (packId, callback) => {
  Package.findByIdAndDelete({ _id: packId }, callback);
};

module.exports.getPackage = callback => {
  Package.find(callback);
};
