const mongoose = require('mongoose');
const config = require('../config/database');

const professionalstatusSchema = mongoose.Schema({
  professional_status_id: { type: String, required: true },
  professional_status_title: { type: String, required: true },
  professional_status_is_deleted: { type: Boolean, },
  professional_status_added_by: { type: String },
  professional_status_order: { type: String },
  professional_status_status: { type: String },
  professional_status_created_on: { type: Date, default: Date.now() }
}, { collection: 'sbw_professional_status' });

const ProfessionalStatus = (module.exports = mongoose.model('sbw_professional_status', professionalstatusSchema));

// Add professionalStatus
module.exports.addProfessionalStatus = (professionalStatus, callback) => {
  ProfessionalStatus.create(professionalStatus, callback);
}

// read professionalStatus
module.exports.getProfessionalStatus = (callback) => {
  ProfessionalStatus.find(callback);
}

//find a professionalStatus by id
module.exports.getProfessionalStatusById = (Id, callback) => {
  ProfessionalStatus.findOne({ "professional_status_id": Id }, callback);
}

//find a professionalStatus by title
module.exports.getProfessionalStatusByTitle = (title, callback) => {
  ProfessionalStatus.findOne({ "professional_status_title": title }, callback);
}


// Update professionalStatus
module.exports.updateProfessionalStatus = (id, professionalStatus, options, callback) => {
  var query = { _id: id };
  var update = {};
  if (professionalStatus.professional_status_title) {
    update.professional_status_title = professionalStatus.professional_status_title
  }
  if (professionalStatus.professional_status_is_deleted) {
    update.professional_status_is_deleted = professionalStatus.professional_status_is_deleted
  }
  if (professionalStatus.professional_status_added_by) {
    update.professional_status_added_by = professionalStatus.professional_status_added_by
  }
  if (professionalStatus.professional_status_order) {
    update.professional_status_order = professionalStatus.professional_status_order
  }
  if (professionalStatus.professional_status_status) {
    update.professional_status_status = professionalStatus.professional_status_status
  }
  ProfessionalStatus.findOneAndUpdate(query, { $set: update }, options, callback);
}


// Delete ProfessionalStatus
module.exports.removeProfessionalStatus = (id, callback) => {
  var query = { professional_status_id: id };
  ProfessionalStatus.remove(query, callback);
}


