const express = require('express');
const router = express.Router();
const passport = require('passport');
const config = require('../config/database');
const FollowUpType = require('../models/follow-up-type');
const OldFollowupTypes = require('../models/old-follow-up-type');

// create followup type
router.post(
  '/createFollowUpType',
  passport.authenticate('jwt', { session: false }),
  (req, res) => {
    FollowUpType.getFollowUpTypeByName(
      req.body.followUpType,
      (err, followUpType) => {
        if (err) {
          res.json({ success: false, msg: JSON.stringify(err) });
        } else {
          if (followUpType) {
            // console.log(FollowUpType)
            return res.json({
              success: false,
              msg: `Follow Up type already exist.`,
              err: err
            });
          } else {
            let newFollowUpType = new FollowUpType({
              followUpType: req.body.followUpType,
              label: req.body.label,
              priority: req.body.priority,
              oldNames: req.body.oldNames,
              icon: req.body.icon,
              hasDueDate: req.body.hasDueDate,
              createdBy: req.body.user
            });

            FollowUpType.addFollowUpType(
              newFollowUpType,
              (err, followUpType) => {
                if (err) {
                  res.status(500).json({
                    success: false,
                    msg: 'Failed to add follow up type',
                    err: JSON.stringify(err)
                  });
                } else {
                  res.status(201).json({
                    success: true,
                    followUpType: followUpType,
                    msg: 'Follow Up Type Added'
                  });
                }
              }
            );
          }
        }
      }
    );
  }
);

// get followup types
router.get(
  '/getFollowUpTypes',
  passport.authenticate('jwt', { session: false }),
  (req, res) => {
    FollowUpType.getFollowUpTypes((err, followuptypes) => {
      if (err) {
        res.json({
          success: false,
          msg: `Follow Up type already exist.`,
          err: err
        });
      } else {
        res.json({
          success: true,
          followuptypes: followuptypes,
          msg: 'Follow up types fetched.'
        });
      }
    });
  }
);

// edit folloup types
router.post('/edit-folloupType', (req, res) => {
  const ft = req.body.FollowupType;
  // console.log(ft);
  FollowUpType.getFollowUpTypeByName(ft.followUpType, (err, foundFolloupType) => {
    if (err) {
      res.json({ success: false, msg: 'Error in updating followup types ', err: JSON.stringify(err) });
    } else {
      // if (foundFolloupType) {
      // res.json({ success: false, msg: 'Followup type already exists' });
      // } else {
      FollowUpType.findById(ft._id, (err, foundFolloupType2) => {
        if (err) {
          res.json({ success: false, msg: 'Error in updating followup types ', err: JSON.stringify(err) });
        } else {
          if (foundFolloupType2) {
            foundFolloupType2.hasDueDate = ft.hasDueDate;
            foundFolloupType2.followUpType = ft.followUpType;
            foundFolloupType2.label = ft.label;
            foundFolloupType2.priority = ft.priority;
            foundFolloupType2.oldNames = ft.oldNames;
            foundFolloupType2.icon = ft.icon;
            // console.log(foundFolloupType);
            FollowUpType.addFollowUpType(foundFolloupType2, (err, updatedFollowupType) => {
              if (err) {
                res.json({ success: false, msg: 'error in updating folloup type', err: JSON.stringify(err) });
              } else {
                res.json({ success: true, msg: 'Folloup type updated successfully' });
              }
            });
          } else {
            res.json({ success: false, msg: 'Followup type not found' });
          }
        }
      });
      // }
    }
  });
});

// delete followup types
router.post(
  '/deleteFollowUpTypes',
  passport.authenticate('jwt', { session: false }),
  (req, res) => {
    // console.log(req.body.followuptype);
    // console.log("----");
    FollowUpType.getFollowUpTypeByName(req.body.followuptype.followUpType, (err, foundFollowUpType) => {
      if (err) {
        res.status(200).json({
          success: false,
          msg: 'Error occured',
          err: JSON.stringify(err)
        });
      } else {
        // console.log("----");
        // console.log(foundFollowUpType)
        if (foundFollowUpType.canDelete) {
          let newfollowUpType = new OldFollowupTypes({
            followUpType: foundFollowUpType.followUpType,
            hasDueDate: foundFollowUpType.hasDueDate,
            createdBy: foundFollowUpType.createdBy,
            createdAt: foundFollowUpType.createdAt,
            updatedBy: req.body.username
          });
          OldFollowupTypes.addFollowUpType(newfollowUpType, (err, followUpType) => {
            if (err) {
              res
                .status(200)
                .json({ success: false, msg: 'error occured', err: err });
            } else {
              FollowUpType.deleteFollowUpType(followUpType.followUpType, (err, deletedfollowUpType) => {
                if (err) {
                  res.status(200).json({
                    success: false,
                    msg: 'error occured',
                    err: err
                  });
                } else {
                  res.status(200).json({
                    success: true,
                    msg: 'Followup type deleted',
                    followUpType: deletedfollowUpType
                  });
                }
              }
              );
            }
          }
          );
        } else {
          res.json({ success: false, msg: 'This followuptype can not be deleted', followupType: foundFollowUpType });
        }
      }
    }
    );
  }
);

module.exports = router;


