const express = require('express');
const router = express.Router();
const IncomeRange = require('../models/income-range');

router.post('/createIncomeRange', (req, res) => {
    IncomeRange.getIncomeRangeByName(req.body.name, (err, irange) => {
        if (err) {
            res.json({ success: false, msg: 'Error in fetching income range by name' });
        } else {
            if (irange) {
                res.json({ success: false, msg: 'Income range already exist with this name.' });
            } else {

                IncomeRange.getIncomeRangeByLowHigh(req.body.low, req.body.high, (err, drange) => {
                    if (err) {
                        res.json({ success: false, msg: 'Error in fetching income range by low high value', err: JSON.stringify(err) });
                    } else {
                        if (drange) {
                            res.json({ success: false, msg: 'Income range already exist with this range.' });
                        } else {
                            let newIR = new IncomeRange({
                                name: req.body.name,
                                // shortName: req.body.shortName,
                                low: parseInt(req.body.low),
                                high: parseInt(req.body.high)
                            });

                            IncomeRange.addIncomeRange(newIR, (err, createdIR) => {
                                if (err) {
                                    res.json({ success: false, msg: 'Error in creating income range', err: JSON.stringify(err) });
                                } else {
                                    res.json({ success: true, msg: 'Income range added successfully', incomeRange: createdIR });
                                }
                            });
                        }
                    }
                });
            }
        }
    });
});

router.post('/getIncomeInRange', (req, res) => {
    const low = req.body.low;
    const high = req.body.high;

    IncomeRange.getInRange(low, high, (err, irange) => {
        if (err) {
            res.json({ success: false, msg: 'Error in getting income range', err: JSON.stringify(err) });
        } else {
            res.json({ success: true, msg: 'Income range found successfully', incomeRange: irange });
        }
    });
});

router.get('/getAllIncomeRanges', (req, res) => {
    IncomeRange.getAllIncomeRange((err, ranges) => {
        if (err) {
            res.json({ success: false, msg: 'Error in fetching income by id', err: JSON.stringify(err) });
        } else {
            res.json({ success: true, msg: 'Income ranges found', incomeRanges: ranges })
        }
    });
});

router.post('/editIncomeRange', (req, res) => {
    IncomeRange.getIncomeRangeById(req.body.id, (err, incomeRange) => {
        if (err) {
            res.json({ success: false, msg: 'Error in fetching income by id', err: JSON.stringify(err) });
        } else {
            if (incomeRange) {
                incomeRange.low = req.body.low;
                incomeRange.high = req.body.high;
                incomeRange.name = req.body.name;
                // incomeRange.shortName = req.body.shortName;

                IncomeRange.addIncomeRange(incomeRange, (err, updatedIR) => {
                    if (err) {
                        res.json({ success: false, msg: 'Error in updating income range', err: JSON.stringify(err) });
                    } else {
                        res.json({ success: true, msg: 'Income range updated' });
                    }
                });

            } else {
                res.json({ success: false, msg: 'Income range not found' });
            }
        }
    });
});


router.post('/deleteIncomeRange', (req, res) => {
    if (err) {
        res.json({ success: false, msg: 'Error in fetching income by id', err: JSON.stringify(err) });
    } else {
        IncomeRange.deleteIncomeRange(req.body.id, (err, deletedIR) => {
            if (err) {
                res.json({ success: false, msg: 'Error in deleting income', err: JSON.stringify(err) });
            } else {
                if (deletedIR) {
                    res.json({ success: true, msg: 'Income range deleted', incomeRange: deletedIR });
                } else {
                    res.json({ success: false, msg: 'Income range not found' });
                }
            }
        });
    }
});

module.exports = router;