const express = require('express');
const router = express.Router();
const passport = require('passport');
const Area = require('../models/area');
const config = require('../config/database');
const Package = require('../models/leadpackage');
const OldLeadPacks = require('../models/oldLeadPackage');
const _ = require('lodash');

router.post('/getPackageById', (req, res) => {
  const packageid = req.body.packageid;
  Package.getPackageById(packageid, (err, package) => {
    if (err) {
      res.json({ success: true, msg: JSON.stringify(err) });
    } else {
      if (package) {
        res.json({ success: true, msg: 'Package Fetched', package: package });
      } else {
        res.json({ success: false, msg: 'Package not found' });
      }
    }
  });
});

// create package
router.post(
  '/createPackage',
  passport.authenticate('jwt', { session: false }),
  (req, res) => {
    Package.getPackageByName(req.body.packageName, (err, package) => {
      if (err) {
        res.json({ success: false, msg: JSON.stringify(err) });
      } else {
        if (package) {
          return res.json({ success: false, msg: `Package already exist.` });
        } else {

          let selectionType = req.body.selectionType;
          let selectedOpt = req.body.selectedOpt;
          let packcities = [];
          // console.log(selectedOpt, selectionType);
          if (selectionType == 'countrywise') {
            for (let i = 0; i < selectedOpt.length; i++) {
              Area.getCitiesByCountry(selectedOpt[i], (err, ffcities) => {
                if (err) {
                  res.json({ success: false, msg: 'Error in fetching cities of selecedOption', err: JSON.stringify(err) });
                } else {
                  let fcities = _.uniqBy(ffcities, (e) => e.city);
                  for (let i = 0; i < fcities.length; i++) {
                    if (fcities[i].city) {
                      packcities.push(fcities[i]);
                    }
                  }
                  if (i == (selectedOpt.length - 1)) {
                    makePackage(packcities, req, res);
                  }
                  // console.log(packcities);
                }
              });
            }
          } else if (selectionType == 'statewise') {
            for (let i = 0; i < selectedOpt.length; i++) {
              Area.getCitiesByState(selectedOpt[i], (err, fcities) => {
                if (err) {
                  res.json({ success: false, msg: 'Error in fetching cities of selecedOption', err: JSON.stringify(err) });
                } else {
                  let uniqCities = [];
                  for (let i = 0; i < fcities.length; i++) {
                    uniqCities.push({
                      city: fcities[i]._id,
                      district: fcities[i].district[0],
                      state: fcities[i].state[0],
                      country: fcities[i].country[0]
                    });
                  }
                  for (let i = 0; i < uniqCities.length; i++) {
                    if (uniqCities[i].city) {
                      packcities.push(uniqCities[i]);
                    }
                  }
                  if (i == (selectedOpt.length - 1)) {
                    makePackage(packcities, req, res);
                  }
                  // console.log("Package Cities::" + packcities);
                }
              });
            }
          } else if (selectionType == 'districtwise') {
            for (let i = 0; i < selectedOpt.length; i++) {
              Area.getCitiesByDistrict(selectedOpt[i], (err, fcities) => {
                if (err) {
                  res.json({ success: false, msg: 'Error in fetching cities of selecedOption', err: JSON.stringify(err) });
                } else {

                  let uniqCities = [];
                  for (let i = 0; i < fcities.length; i++) {
                    uniqCities.push({
                      city: fcities[i]._id,
                      district: fcities[i].district[0],
                      state: fcities[i].state[0],
                      country: fcities[i].country[0]
                    });
                  }

                  for (let i = 0; i < uniqCities.length; i++) {
                    if (uniqCities[i].city) {
                      packcities.push(uniqCities[i]);
                    }
                  }

                  if (i == (selectedOpt.length - 1)) {
                    makePackage(packcities, req, res);
                  }
                  // console.log("cities: " + packcities);
                }
              });
            }
          } else if (selectionType == 'citywise') {
            packcities = selectedOpt;
            makePackage(packcities, req, res);
            // console.log(packcities);
          }
        }
      }
    });
  }
);

function makePackage(packcities, req, res) {
  if (packcities.length) {
    let newPackage = new Package({
      packageName: req.body.packageName,
      shortName: req.body.shortName,
      packageAmount: req.body.packageAmount,
      balance: req.body.balance,
      createdBy: req.body.user,
      purpose: req.body.purpose,
      profileLimit: req.body.profileLimit,
      source: req.body.source,
      serviceType: req.body.serviceType,
      incomeRange: req.body.incomeRange,
      cities: packcities,
      unit: req.body.unit,
      rate: req.body.rate,
      discount: req.body.discount,
      quantity: req.body.quantity,
      additionalCharges: req.body.additionalCharges,
      packageType: req.body.packageType,
      renewDate: req.body.renewDate,
      serviceCount: req.body.serviceCount,
      serviceGap: req.body.serviceGap,
      billingCount: req.body.billingCount,
      billingGap: req.body.billingGap,
    });

    Package.addPackage(newPackage, (err, package) => {
      if (err) {
        // // // console.log(err);
        res.status(500).json({
          success: false,
          msg: 'Failed to add Package',
          err: JSON.stringify(err)
        });
      } else {
        // // // console.log(package)
        res.status(201).json({
          success: true,
          package: package,
          msg: 'Package is added'
        });
      }
    });
  } else {
    res.json({ success: false, msg: 'Failed to fetch cities, No cities found6' });
  }
}

// get pacakges
router.get(
  '/getPackages',
  passport.authenticate('jwt', { session: false }),
  (req, res) => {
    Package.getPackage((err, packages) => {
      if (err) {
        res
          .status(200)
          .json({ success: false, msg: 'error occured', err: err });
      } else {
        res
          .status(200)
          .json({ success: true, msg: 'packages found', packages: packages });
      }
    });
  }
);

// get Field pacakges
router.post(
  '/getFieldPackages',
  passport.authenticate('jwt', { session: false }),
  (req, res) => {
    // console.log(req.body.source, req.body.purpose, req.body.cities);
    Package.getPackagesByFields(req.body.source, req.body.purpose, req.body.cities, (err, packages) => {
      //   console.log(packages);
      if (err) {
        res
          .status(200)
          .json({ success: false, msg: 'error occured', err: err });
      } else {
        res
          .status(200)
          .json({ success: true, msg: 'packages found', packages: packages });
      }
    });
  }
);

// edit package
router.post(
  '/editPackage',
  passport.authenticate('jwt', { session: false }),
  (req, res) => {
    let package = req.body.package;
    Package.getPackageById(package._id, (err, pack) => {
      if (err) {
        res
          .status(200)
          .json({ success: false, msg: 'error occured', err: err });
      } else {
        if (pack) {
          pack.cities = package.cities;
          pack.purpose = package.purpose;
          pack.source = package.source;
          pack.serviceType = package.serviceType;
          pack.packageName = package.packageName;
          pack.profileLimit = package.profileLimit;
          pack.packageAmount = package.advance;
          pack.incomeRange = package.incomeRange;
          pack.balance = package.balance;
          pack.unit = package.unit;
          pack.rate = package.rate;
          pack.discount = package.discount;
          pack.quantity = package.quantity;
          pack.additionalCharges = package.additionalCharges;
          pack.packageType = package.packageType;
          pack.renewDate = package.renewDate;
          pack.serviceCount = package.serviceCount;
          pack.serviceGap = package.serviceGap;
          pack.billingCount = package.billingCount;
          pack.billingGap = package.billingGap;

          Package.addPackage(pack, (err, pack) => {
            if (err) {
              res
                .status(200)
                .json({ success: false, msg: 'error occured', err: err });
            } else {
              res.status(200).json({
                success: true,
                msg: 'Package edit successful ',
                package: pack
              });
            }
          });
        } else {
          res.status(200).json({ success: false, msg: 'package not found' });
        }
      }
    });
  }
);

router.post('/delete-leadPackage', passport.authenticate('jwt', { session: false }), (req, res) => {
  // console.log(req.body.packageName)
  Package.getPackageByName(req.body.packageName, (err, pack) => {
    if (err) {
      res.json({ success: false, msg: 'error occured' });
    } else {
      if (pack) {
        let newPack = new OldLeadPacks({
          packageName: pack.packageName,
          shortName: pack.shortName,
          packageAmount: pack.packageAmount,
          createdBy: pack.user,
          purpose: pack.purpose,
          profileLimit: pack.profileLimit,
          source: pack.source,
          serviceType: pack.serviceType,
          cities: pack.cities,
          unit: pack.unit,
          rate: pack.rate,
          discount: pack.discount,
          quantity: pack.quantity,
          additionalCharges: pack.additionalCharges,
          packageType: pack.packageType,
          renewDate: pack.renewDate,
          serviceCount: pack.serviceCount,
          serviceGap: pack.serviceGap,
          billingCount: pack.billingCount,
          billingGap: pack.billingGap,
        });
        OldLeadPacks.addPackage(newPack, (err, addedPck) => {
          if (err) {
            res.json({ success: false, msg: 'Error in deletion package' });
          } else {
            if (addedPck) {
              // console.log(addedPck)
              Package.deletePackage(pack._id, (err, deletedPack) => {
                if (err) {
                  res.json({ success: false, msg: 'error in deleting package' });
                } else {
                  res.json({ success: true, msg: 'Package is deleted successfully' });
                }
              });
            } else {
              res.json({ success: false, msg: 'Error in deletion package' });
            }
          }
        });
      } else {
        res.json({ success: false, msg: 'package not found' });
      }
    }
  });
});



// create package
router.post(
  '/edit-Package',
  passport.authenticate('jwt', { session: false }),
  (req, res) => {
    // console.log("Edit Package");
    Package.getPackageById(req.body._id, (err, package) => {
      // console.log(package);
      if (err) {
        res.json({ success: false, msg: JSON.stringify(err) });
      } else {
        if (!package) {
          return res.json({ success: false, msg: `Package doesnt exist.` });
        } else {

          let selectionType = req.body.selectionType;
          let selectedOpt = req.body.selectedOpt;
          let packcities = [];
          // console.log(selectedOpt, selectionType);
          if (selectionType == 'countrywise') {
            for (let i = 0; i < selectedOpt.length; i++) {
              Area.getCitiesByCountry(selectedOpt[i], (err, ffcities) => {
                if (err) {
                  res.json({ success: false, msg: 'Error in fetching cities of selecedOption', err: JSON.stringify(err) });
                } else {
                  let fcities = _.uniqBy(ffcities, (e) => e.city);
                  for (let i = 0; i < fcities.length; i++) {
                    if (fcities[i].city) {
                      packcities.push(fcities[i]);
                    }
                  }
                  if (i == (selectedOpt.length - 1)) {
                    recreatePackage(package, packcities, req, res);
                  }
                  // console.log(packcities);
                }
              });
            }
          } else if (selectionType == 'statewise') {
            for (let i = 0; i < selectedOpt.length; i++) {
              Area.getCitiesByState(selectedOpt[i], (err, fcities) => {
                if (err) {
                  res.json({ success: false, msg: 'Error in fetching cities of selecedOption', err: JSON.stringify(err) });
                } else {
                  let uniqCities = [];
                  for (let i = 0; i < fcities.length; i++) {
                    uniqCities.push({
                      city: fcities[i]._id,
                      district: fcities[i].district[0],
                      state: fcities[i].state[0],
                      country: fcities[i].country[0]
                    });
                  }
                  for (let i = 0; i < uniqCities.length; i++) {
                    if (uniqCities[i].city) {
                      packcities.push(uniqCities[i]);
                    }
                  }
                  if (i == (selectedOpt.length - 1)) {
                    recreatePackage(package, packcities, req, res);
                  }
                  // console.log("Package Cities::" + packcities);
                }
              });
            }
          } else if (selectionType == 'districtwise') {
            for (let i = 0; i < selectedOpt.length; i++) {
              Area.getCitiesByDistrict(selectedOpt[i], (err, fcities) => {
                if (err) {
                  res.json({ success: false, msg: 'Error in fetching cities of selecedOption', err: JSON.stringify(err) });
                } else {

                  let uniqCities = [];
                  for (let i = 0; i < fcities.length; i++) {
                    uniqCities.push({
                      city: fcities[i]._id,
                      district: fcities[i].district[0],
                      state: fcities[i].state[0],
                      country: fcities[i].country[0]
                    });
                  }

                  for (let i = 0; i < uniqCities.length; i++) {
                    if (uniqCities[i].city) {
                      packcities.push(uniqCities[i]);
                    }
                  }

                  if (i == (selectedOpt.length - 1)) {
                    recreatePackage(package, packcities, req, res);
                  }
                  // console.log("cities: " + packcities);
                }
              });
            }
          } else if (selectionType == 'citywise') {
            packcities = selectedOpt;
            recreatePackage(package, packcities, req, res);
            // console.log(packcities);
          } else {
            recreatePackage(package, req.body.cities, req, res);
          }
        }
      }
    });
  }
);


function recreatePackage(package, packcities, req, res) {
  if (packcities.length) {
    // let newPackage = new Package({
    //   _id: req.body._id,
    package.packageName = req.body.packageName;
    package.shortName = req.body.shortName;
    package.packageAmount = req.body.packageAmount;
    package.balance = req.body.balance;
    package.createdBy = req.body.user;
    package.purpose = req.body.purpose;
    package.profileLimit = req.body.profileLimit;
    package.source = req.body.source;
    package.serviceType = req.body.serviceType;
    package.incomeRange = req.body.incomeRange;
    package.cities = packcities;
    package.unit = req.body.unit;
    package.rate = req.body.rate;
    package.discount = req.body.discount;
    package.quantity = req.body.quantity;
    package.additionalCharges = req.body.additionalCharges;
    package.packageType = req.body.packageType;
    package.renewDate = req.body.renewDate;
    package.serviceCount = req.body.serviceCount;
    package.serviceGap = req.body.serviceGap;
    package.billingCount = req.body.billingCount;
    package.billingGap = req.body.billingGap;
    // });

    Package.addPackage(package, (err, package) => {
      if (err) {
        // // // console.log(err);
        res.status(500).json({
          success: false,
          msg: 'Failed to add Package',
          err: JSON.stringify(err)
        });
      } else {
        // // // console.log(package)
        res.status(201).json({
          success: true,
          package: package,
          msg: 'Package Edited Successfully'
        });
      }
    });
  } else {
    res.json({ success: false, msg: 'Failed to fetch cities, No cities found6' });
  }
}

module.exports = router;
