const express = require('express');
const router = express.Router();
const PrivilageCoupen = require('../models/privilageCoupen');

router.post('/createPrivilageCoupen', (req, res) => {
    PrivilageCoupen.getPrivilageCoupenByShortName(req.body.shortName, (err, icoupen) => {
        if (err) {
            res.json({ success: false, msg: 'Error in fetching privilage coupon by name' });
        } else {
            if (icoupen) {
                res.json({ success: false, msg: 'Privilage coupon already exist with this short name.' });
            } else {

                PrivilageCoupen.getPrivilageCoupenByLowHigh(req.body.low, req.body.high, (err, dcoupen) => {
                    if (err) {
                        res.json({ success: false, msg: 'Error in fetching privilage coupon by low high value', err: JSON.stringify(err) });
                    } else {
                        if (dcoupen) {
                            res.json({ success: false, msg: 'Privilage coupon already exist with this coupon.' });
                        } else {
                            let newIR = new PrivilageCoupen({
                                name: req.body.name,
                                shortName: req.body.shortName,
                                description: req.body.description,
                                discount: req.body.discount,
                                low: parseInt(req.body.low),
                                high: parseInt(req.body.high)
                            });

                            PrivilageCoupen.addPrivilageCoupen(newIR, (err, createdIR) => {
                                if (err) {
                                    res.json({ success: false, msg: 'Error in creating privilage coupon', err: JSON.stringify(err) });
                                } else {
                                    res.json({ success: true, msg: 'Privilage coupon added successfully', privilageCoupen: createdIR });
                                }
                            });
                        }
                    }
                });
            }
        }
    });
});

router.post('/getPrivilageCouponById', (req, res) => {
    const id = req.body.id;

    PrivilageCoupen.getPrivilageCoupenById(id, (err, icoupen) => {
        if (err) {
            res.json({ success: false, msg: 'Error in getting privilage coupon', err: JSON.stringify(err) });
        } else {
            res.json({ success: true, msg: 'Privilage coupon found successfully', privilageCoupen: icoupen });
        }
    });
});

router.post('/getPrivilageInRange', (req, res) => {
    const low = req.body.low;
    const high = req.body.high;

    PrivilageCoupen.getInRange(low, high, (err, icoupen) => {
        if (err) {
            res.json({ success: false, msg: 'Error in getting privilage coupon', err: JSON.stringify(err) });
        } else {
            res.json({ success: true, msg: 'Privilage coupon found successfully', privilageCoupen: icoupen });
        }
    });
});

router.get('/getAllPrivilageCoupens', (req, res) => {
    PrivilageCoupen.getAllPrivilageCoupen((err, coupens) => {
        if (err) {
            res.json({ success: false, msg: 'Error in fetching privilage by id', err: JSON.stringify(err) });
        } else {
            res.json({ success: true, msg: 'Privilage coupons found', privilageCoupens: coupens })
        }
    });
});

router.post('/editPrivilageCoupen', (req, res) => {
    PrivilageCoupen.getPrivilageCoupenById(req.body._id, (err, privilageCoupen) => {
        if (err) {
            res.json({ success: false, msg: 'Error in fetching privilage by id', err: JSON.stringify(err) });
        } else {
            if (privilageCoupen) {
                privilageCoupen.low = req.body.low;
                privilageCoupen.high = req.body.high;
                privilageCoupen.name = req.body.name;
                privilageCoupen.discount = req.body.discount;
                privilageCoupen.shortName = req.body.shortName;
                privilageCoupen.description = req.body.description;

                PrivilageCoupen.addPrivilageCoupen(privilageCoupen, (err, updatedIR) => {
                    if (err) {
                        res.json({ success: false, msg: 'Error in updating privilage coupon', err: JSON.stringify(err) });
                    } else {
                        res.json({ success: true, msg: 'Privilage coupon updated' });
                    }
                });

            } else {
                res.json({ success: false, msg: 'Privilage coupon not found' });
            }
        }
    });
});


router.post('/deletePrivilageCoupen', (req, res) => {
    if (err) {
        res.json({ success: false, msg: 'Error in fetching privilage by id', err: JSON.stringify(err) });
    } else {
        PrivilageCoupen.deletePrivilageCoupen(req.body.id, (err, deletedIR) => {
            if (err) {
                res.json({ success: false, msg: 'Error in deleting privilage', err: JSON.stringify(err) });
            } else {
                if (deletedIR) {
                    res.json({ success: true, msg: 'Privilage coupon deleted', privilageCoupen: deletedIR });
                } else {
                    res.json({ success: false, msg: 'Privilage coupon not found' });
                }
            }
        });
    }
});

module.exports = router;