const express = require('express');
const router = express.Router();
const config = require('../config/database');
const Role = require('../models/role');
const passport = require('passport');

router.post(
  '/create-role',
  passport.authenticate('jwt', { session: false }),
  (req, res) => {
    let role = new Role({
      createdBy: req.body.createdBy,
      name: req.body.name,
      remark: req.body.remark,
      topRole: req.body.topRole,
      permissions: req.body.permissions,
    });

    Role.getRolebyName(role.name, (err, foundRole) => {
      if (err) {
        res.json({ success: false, msg: 'Error in fetching role', err: JSON.stringify(err) });
      } else {
        if (foundRole) {
          res.json({ success: false, msg: 'Role already exist with this Role Name.' });
        } else {
            Role.addRole(role, (err, newRole) => {
              if (err) {
                res.status(200).json({ success: false, msg: 'Internal Server Error', err: err });
              } else {
                res.status(200).json({
                  success: true,
                  msg: 'Role added successfully',
                  role: newRole
                });
              }
            });
        }
      }
    });
  }
);

router.get(
  '/getAllRoles',
  passport.authenticate('jwt', { session: false }),
  (req, res) => {
    Role.getAllRoles((err, roleList) => {
      if (err) {
        res.status(200).json({
          success: false,
          msg: 'Failed to fetch role list.',
          err: err
        });
      } else {
        for (const rol of roleList) {
          if (!rol.permissions.ss) {
            if (rol.name === 'super-admin') {
              rol.permissions.ss = {
                enable: true,
                members: {
                  create: true,
                  view: true,
                  edit: true,
                  delete: true,
                  export: true,
                  enable: true
                },
                adminapprove: {
                  // create: true,
                  view: true,
                  edit: true,
                  // delete: true,
                  enable: true
                },
                registeruser: {
                  create: true,
                  // view: true,
                  // edit: true,
                  // delete: true,
                  enable: true
                },
                transaction: {
                  // create: true,
                  view: true,
                  // edit: true,
                  // delete: true,
                  export: true,
                  enable: true
                },
              }
            } else {
              rol.permissions.ss = {
                enable: false,
                members: {
                  create: false,
                  view: false,
                  edit: false,
                  delete: false,
                  export: false,
                  enable: false
                },
                adminapprove: {
                  // create: true,
                  view: false,
                  edit: false,
                  // delete: true,
                  enable: false
                },
                registeruser: {
                  create: false,
                  // view: true,
                  // edit: true,
                  // delete: true,
                  enable: false
                },
                transaction: {
                  // create: true,
                  view: false,
                  // edit: true,
                  // delete: true,
                  export: false,
                  enable: false
                },
              }
            }
          }
        }
        res.status(200).json({
          success: true,
          msg: 'Role list fetched',
          roleList: roleList
        });
      }
    });
  }
);

router.post(
  '/getRoleById',
  passport.authenticate('jwt', { session: false }),
  (req, res) => {
    Role.getRoleById(req.body.id, (err, role) => {
      if (err) {
        res.status(200).json({
          success: false,
          msg: 'Failed to fetch role details',
          err: err
        });
      } else {
        if(role){
          if (!role.permissions.ss) {
            if (role.name === 'super-admin') {
              role.permissions.ss = {
                enable: true,
                members: {
                  create: true,
                  view: true,
                  edit: true,
                  delete: true,
                  export: true,
                  enable: true
                },
                adminapprove: {
                  // create: true,
                  view: true,
                  edit: true,
                  // delete: true,
                  enable: true
                },
                registeruser: {
                  create: true,
                  // view: true,
                  // edit: true,
                  // delete: true,
                  enable: true
                },
                transaction: {
                  // create: true,
                  view: true,
                  // edit: true,
                  // delete: true,
                  export: true,
                  enable: true
                },
              }
            } else {
              role.permissions.ss = {
                enable: false,
                members: {
                  create: false,
                  view: false,
                  edit: false,
                  delete: false,
                  export: false,
                  enable: false
                },
                adminapprove: {
                  // create: true,
                  view: false,
                  edit: false,
                  // delete: true,
                  enable: false
                },
                registeruser: {
                  create: false,
                  // view: true,
                  // edit: true,
                  // delete: true,
                  enable: false
                },
                transaction: {
                  // create: true,
                  view: false,
                  // edit: true,
                  // delete: true,
                  export: false,
                  enable: false
                },
              }
            }
          }
          res
          .status(200)
          .json({ success: true, msg: 'role fetched', role: role });
        } else {
          res.json({ success: false,  msg: 'Role not found'});
        }
      }
    });
  }
);

router.post(
  '/getRolePermissions',
  passport.authenticate('jwt', { session: false }),
  (req, res) => {
    Role.getRolePermissions(req.body.role, (err, role) => {
      if (err) {
        res.status(200).json({
          success: false,
          msg: 'Failed to fetch role permissions',
          err: err
        });
      } else {
        if(role){
          if (!role.permissions.ss) {
            if (role.name === 'super-admin') {
              role.permissions.ss = {
                enable: true,
                members: {
                  create: true,
                  view: true,
                  edit: true,
                  delete: true,
                  export: true,
                  enable: true
                },
                adminapprove: {
                  // create: true,
                  view: true,
                  edit: true,
                  // delete: true,
                  enable: true
                },
                registeruser: {
                  create: true,
                  // view: true,
                  // edit: true,
                  // delete: true,
                  enable: true
                },
                transaction: {
                  // create: true,
                  view: true,
                  // edit: true,
                  // delete: true,
                  export: true,
                  enable: true
                },
              }
            } else {
              role.permissions.ss = {
                enable: false,
                members: {
                  create: false,
                  view: false,
                  edit: false,
                  delete: false,
                  export: false,
                  enable: false
                },
                adminapprove: {
                  // create: true,
                  view: false,
                  edit: false,
                  // delete: true,
                  enable: false
                },
                registeruser: {
                  create: false,
                  // view: true,
                  // edit: true,
                  // delete: true,
                  enable: false
                },
                transaction: {
                  // create: true,
                  view: false,
                  // edit: true,
                  // delete: true,
                  export: false,
                  enable: false
                },
              }
            }
          }
          res
          .status(200)
          .json({ success: true, msg: 'role fetched', role: role });
        } else {
          res.json({ success: false,  msg: 'Role not found'});
        }
      }
    });
  }
);


router.post('/edit-role', passport.authenticate('jwt', { session: false }), (req, res) => {
  let role = req.body.role;
//   console.log(role);
  Role.getRoleById(role._id, (err, foundRole) => {
    if (err) {
      res.json({ success: false, msg: 'Error in updating role', err: JSON.stringify(err) });
    } else {
      if (foundRole) {
        foundRole.name = role.name;
        foundRole.remark = role.remark;
        foundRole.topRole = role.topRole;
        foundRole.permissions = role.permissions;
        foundRole.updatedBy = role.updatedBy;
        foundRole.updatedAt = Date.now();

        Role.addRole(foundRole, (err, updatedRole) => {
          if (err) {
            res.json({ success: false, msg: 'Error in editing role', err: JSON.stringify(err) });
          } else {
            res.json({ success: true, msg: 'Role edited successfully', role: updatedRole });
          }
        });
      } else {
        res.json({ success: false, msg: 'role not found' });
      }
    }
  });
});



router.post('/delete-role', passport.authenticate('jwt', { session: false }), (req, res) => {
  Role.getRolebyName(req.body.roleName, (err, foundRole) => {
    if (err) {
      res.json({ success: false, msg: 'Error in deletion of role', err: JSON.stringify(err) });
    } else {
      if (foundRole) {
        Role.deleteRole(foundRole._id, (err, deletedRole) => {
          if (err) {
            res.json({ success: false, msg: 'Error in deletion of role', err: JSON.stringify(err) });
          } else {
            res.json({ success: true, msg: 'Role deleted successfully ' });
          }
        });
      } else {
        res.json({ success: false, msg: 'Role not found' });
      }
    }
  });
});

module.exports = router;
