const express = require('express');
const router = express.Router();
const config = require('../config/database');
const Source = require('../models/source');
const passport = require('passport');

router.post(
  '/create-source',
  passport.authenticate('jwt', { session: false }),
  (req, res) => {
    let source = new Source({
      createdBy: req.body.createdBy,
      sourceName: req.body.sourceName,
      shortName: req.body.shortName,
      show: req.body.show
    });

    Source.getSourcebyName(source.sourceName, (err, foundSource) => {
      if (err) {
        res.json({ success: false, msg: 'Error in fetching source', err: JSON.stringify(err) });
      } else {
        if (foundSource) {
          res.json({ success: false, msg: 'Source already exist with this sourceName.' });
        } else {
          Source.getSourcebyShortname(req.body.shortName, (err, ffoundSource) => {
            if (err) {
              res.json({ success: false, msg: 'Error in fetching source', err: JSON.stringify(err) });
            } else {
              if (ffoundSource) {
                res.json({ success: false, msg: 'Source already exist with this shortName' });
              } else {
                Source.addSource(source, (err, newSource) => {
                  if (err) {
                    res.status(200).json({ success: false, msg: '', err: err });
                  } else {
                    res.status(200).json({
                      success: true,
                      msg: 'Source added successfully',
                      source: newSource
                    });
                  }
                });
              }
            }
          });
        }
      }
    });
  }
);

router.get(
  '/getAllSource',
  passport.authenticate('jwt', { session: false }),
  (req, res) => {
    Source.getAllSources((err, sourceList) => {
      if (err) {
        res.status(200).json({
          success: false,
          msg: 'Failed to fetch source list.',
          err: err
        });
      } else {
        res.status(200).json({
          success: true,
          msg: 'Source list fetched',
          sourceList: sourceList
        });
      }
    });
  }
);

router.post(
  '/getSourceById',
  passport.authenticate('jwt', { session: false }),
  (req, res) => {
    Source.getSourceById(req.body.sourceId, (err, source) => {
      if (err) {
        res.status(200).json({
          success: false,
          msg: 'Failed to fetch source details',
          err: err
        });
      } else {
        if(source){
          res
          .status(200)
          .json({ success: true, msg: 'source fetched', source: source });
        } else {
          res.json({ success: false,  msg: 'Source not found'});
        }
      }
    });
  }
);


router.post('/edit-source', passport.authenticate('jwt', { session: false }), (req, res) => {
  let source = req.body.source;
  console.log(source);
  Source.getSourceById(source._id, (err, foundSource) => {
    if (err) {
      res.json({ success: false, msg: 'Error in updating source', err: JSON.stringify(err) });
    } else {
      if (foundSource) {
        foundSource.sourceName = source.sourceName;
        foundSource.sourceCode = source.sourceCode;
        foundSource.sourceId = source.sourceId;
        foundSource.shortName = source.shortName;
        foundSource.show = source.show;

        Source.addSource(foundSource, (err, updatedSource) => {
          if (err) {
            res.json({ success: false, msg: 'Error in editing source', err: JSON.stringify(err) });
          } else {
            res.json({ success: true, msg: 'Source edited successfully', source: updatedSource });
          }
        });
      } else {
        res.json({ success: false, msg: 'source not found' });
      }
    }
  });
});



router.post('/delete-source', passport.authenticate('jwt', { session: false }), (req, res) => {
  Source.getSourcebyName(req.body.sourceName, (err, foundSource) => {
    if (err) {
      res.json({ success: false, msg: 'Error in deletion of source', err: JSON.stringify(err) });
    } else {
      if (foundSource) {
        Source.deleteSource(foundSource.sourceName, (err, deletedSource) => {
          if (err) {
            res.json({ success: false, msg: 'Error in deletion of source', err: JSON.stringify(err) });
          } else {
            res.json({ success: true, msg: 'Source deleted successfully ' });
          }
        });
      } else {
        res.json({ success: false, msg: 'Source not found' });
      }
    }
  });
});

module.exports = router;
