const express = require('express');
const router = express.Router();
const passport = require('passport');
const SSNotification = require('../models/ss_notification');

router.post(
    '/createSSNotification',
    (req, res) => {
        const newSS = new SSNotification({
            notificationType: req.body.notificationType,
            notificationData: req.body.notificationData,
            title: req.body.title,
            msg: req.body.msg,
            enable: true,
            read: false,
            createdBy: 'sbw-admin',
            createdAt: Date.now(),
            updatedAt: Date.now()
        });
        SSNotification.addSSNotification(newSS, (err, ssNotification) => {
            if (err) {
                res
                    .status(200)
                    .json({ success: false, msg: 'Something went wrong', err: err });
            } else {
                res
                    .status(200)
                    .json({ success: true, msg: 'notification created', data: ssNotification });

            }
        });
    }
);

router.post(
    '/getSSNotification/:id',
    (req, res) => {
        const id = req.params.id;
        const member_id = req.body.member_id;
        SSNotification.getSSNotificationsByMemberId(id, member_id, (err, ssNotification) => {
            if (err) {
                res
                    .status(200)
                    .json({ success: false, msg: 'Something went wrong', err: err });
            } else {
                if (ssNotification) {
                    res
                        .status(200)
                        .json({ success: true, msg: 'notification are found', data: ssNotification });
                } else {
                    res
                        .status(200)
                        .json({ success: false, msg: 'notification not found', data: ssNotification });
                }
            }
        });
    }
);

router.delete(
    '/delete/:id',
    (req, res) => {
        const id = req.params.id;
        SSNotification.disableNotification(id, (err, ssNotification) => {
            if (err) {
                res
                    .status(200)
                    .json({ success: false, msg: 'Something went wrong', err: err });
            } else {
                if (ssNotification) {
                    res
                        .status(200)
                        .json({ success: true, msg: 'notification deleted', data: ssNotification });
                } else {
                    res
                        .status(200)
                        .json({ success: false, msg: 'notification not found', data: ssNotification });
                }
            }
        });
    }
);

router.get(
    '/getAllNotifications/:skip',
    (req, res) => {
        const skip = (req.params.skip * 2000) - 2000;
        SSNotification.getAllSSNotifications(skip, (err, notifications) => {
            if (err) {
                res
                    .status(200)
                    .json({ success: false, msg: 'Something went wrong', err: err });
            } else {
                if (notifications.length) {
                    res
                        .status(200)
                        .json({ success: true, msg: 'notifications are found', data: notifications });
                } else {
                    res
                        .status(200)
                        .json({ success: false, msg: 'notifications not found', data: notifications });
                }
            }
        });
    }
);

router.post(
    '/getDisabledNotifications/:skip',
    (req, res) => {
        const skip = (req.params.skip * 2000) - 2000;
        SSNotification.getAllDisabledSSNotifications(skip, (err, notifications) => {
            if (err) {
                res
                    .status(200)
                    .json({ success: false, msg: 'Something went wrong', err: err });
            } else {
                if (notifications.length) {
                    res
                        .status(200)
                        .json({ success: true, msg: 'notifications are found', data: notifications });
                } else {
                    res
                        .status(200)
                        .json({ success: false, msg: 'notifications not found', data: notifications });
                }
            }
        });
    }
);

router.post(
    '/searchNotification/:keyword/:skip',
    passport.authenticate('jwt', { session: false }),
    (req, res) => {
        const searchKeyword = new RegExp(req.params.keyword, 'gi');
        const skip = (req.params.skip * 2000) - 2000;
        SSNotification.searchInSSNotifications(searchKeyword, skip, (err, ssNotifications) => {
            if (err) {
                res
                    .status(200)
                    .json({ success: false, msg: 'Something went wrong', err: err });
            } else {
                if (ssNotifications.length) {
                    res
                        .status(200)
                        .json({ success: true, msg: 'notifications are found', data: ssNotifications });
                } else {
                    res
                        .status(200)
                        .json({ success: false, msg: 'notifications not found', data: ssNotifications });
                }
            }
        });
    }
);

module.exports = router;